//    ATTN:
//     _____              _ _     _    _            _______ _     _
//    |  __ \            ( ) |   | |  | |          |__   __| |   (_)
//    | |  | | ___  _ __ |/| |_  | |  | |___  ___     | |  | |__  _ ___
//    | |  | |/ _ \| '_ \  | __| | |  | / __|/ _ \    | |  | '_ \| / __|
//    | |__| | (_) | | | | | |_  | |__| \__ \  __/    | |  | | | | \__ \
//    |_____/ \___/|_| |_|  \__|  \____/|___/\___|    |_|  |_| |_|_|___/
//
//    This module is not finalized. The functionality contained herein
//    is not finalized. Everything in this file will change in future
//    releases. If you use this module or code, you are guaranteeing
//    the code you write will break in future releases.
//
//    To reiterate: DO NOT USE THIS MODULE.
//                  DO NOT USE THIS CODE.
//

/* global console */
/* global CKEDITOR */
/* global PortalTools */
/* global require */
/* global define */

/**
 * BusyIndicator Module
 * Used to create BusyIndicator Objects
 * @module BusyIndicator
 */
//alert("BusyIndicator");

define(["jquery", "underscore", "domReady"], function($, _, domReady) {
        "use strict";
//alert("BusyIndicator A");

        var root = window,
            defaultOptions = {
                waitTime : 2000
            },
            busyIndicatorObjects = [];

        function BusyIndicator(selector) {
            if(!selector) {
                throw new Error("Selector is required.");
            }

            _.bindAll(this,
                "initialize",
                "start",
                "stop"
            );
            this.selector = selector;
            this.timerId = null;
            this.areasOriginalConfiguration = [];
            domReady(this.initialize);

            busyIndicatorObjects.push(this);
        }
        BusyIndicator.prototype.initialize = function() {
            this.areas = $(this.selector);
        };
        BusyIndicator.prototype.start = function() {
            if(this.timerId || !this.areas) {
                // feedback handler is already running
                return;
            }
            var busyIndicatorObject = this;
            this.areas.each(function() {
                var item = $(this);
                busyIndicatorObject.areasOriginalConfiguration.push({
                    element  : this,
                    disabled : item.prop('disabled')
                });
                item.prop('disabled', true);
            });

            this.timerId = root.setTimeout(function() {
                //TODO: language (needs to grab from server or something)
                busyIndicatorObject.areas.busyBox( "on", "Please wait...", 160, 40, {"initialDelay" : 0} );
            }, defaultOptions.waitTime);
            return this;
        };
        BusyIndicator.prototype.stop = function() {
            if(this.areas) {
                var busyIndicatorObject = this;
                this.areas.each(function () {
                    var storedProperty = _.findWhere(
                        busyIndicatorObject.areasOriginalConfiguration,
                        { element : this }
                    );
                    $(this).prop('disabled', storedProperty ? storedProperty.disabled : null);
                });
                this.areasOriginalConfiguration = [];

                this.areas.busyBox("off");
            }

            if(this.timerId) {
                root.clearTimeout(this.timerId);
                this.timerId = null;
            }
            return this;
        };

//alert("BusyIndicator B");

        return {
            newIndicator : function(selector) {
                return new BusyIndicator(selector);
            },
            start : function(selector) {
                var busyIndicator = new BusyIndicator(selector);
                return busyIndicator.start();
            },
            stopAll : function() {
                _.each(busyIndicatorObjects, function(busyIndicatorObject) {
                    busyIndicatorObject.stop();
                });
            }

        };
    });
