//    ATTN:
//     _____              _ _     _    _            _______ _     _
//    |  __ \            ( ) |   | |  | |          |__   __| |   (_)
//    | |  | | ___  _ __ |/| |_  | |  | |___  ___     | |  | |__  _ ___
//    | |  | |/ _ \| '_ \  | __| | |  | / __|/ _ \    | |  | '_ \| / __|
//    | |__| | (_) | | | | | |_  | |__| \__ \  __/    | |  | | | | \__ \
//    |_____/ \___/|_| |_|  \__|  \____/|___/\___|    |_|  |_| |_|_|___/
//
//    This module is not finalized. The functionality contained herein
//    is not finalized. Everything in this file will change in future
//    releases. If you use this module or code, you are guaranteeing
//    the code you write will break in future releases.
//
//    To reiterate: DO NOT USE THIS MODULE.
//                  DO NOT USE THIS CODE.
//

/* global console */
/* global CKEDITOR */
/* global PortalTools */
/* global require */
/* global define */

/**
 * dialog Module
 * @module BusyIndicator
 */
define(
    [
        "jquery",
        "underscore"
    ],
    function($, _) {
        "use strict";
//alert("dialog A");

        var dialogIdCounter = 0,
            createDialogArea = function() {
                return $('<div/>', {
                    id: 'dialog-box' + (dialogIdCounter++)
                });
            };


        function Dialog(options) {
            _.bindAll(this,
                'close',
                'getDefaultOptions',
                'isOpen',
                'open',
                'updateDialogContentHTML'
            );

            this.dialogArea = createDialogArea();
            this.configuration = this.getDefaultOptions(options.type);

            // convert our options to jQueryUI dialog options
            if(options.buttons) {
                this.configuration.buttons = _.map(options.buttons, function(button) {
                    var configurationObject = {};
                    if(button.text) {
                        configurationObject.text = button.text;
                    }
                    if(button.callbackFunction) {
                        configurationObject.click = _.bind(button.callbackFunction, this);
                    } else {
                        configurationObject.click = function() {};
                    }
                    return configurationObject;
                }, this);
            }

            this.dialogHTML = options.html || '<p>' + options.text + '</p>';

        }
        Dialog.prototype.close = function() {
            var deferred = $.Deferred();
            this.dialogArea.on("dialogclose", this, function(event) {
                var dialog = event.data;
                dialog.dialogArea.remove();
                dialog.dialogArea = null;
                deferred.resolveWith(this);
            });
            this.dialogArea.dialog("close");
            return deferred.promise();
        };
        Dialog.prototype.getDefaultOptions = function(type) {
            switch(type) {
                case "alert":
                case "confirm":
                    return {
                        resizable:   false,
                        dialogClass: "dialog-no-header",
                        minWidth:    450,
                        width: 		 "auto",
                        height:		 "auto",
                        modal:       true,
                        draggable:   false,
                        buttons: {
                            "OK": this.close
                        }
                    };
                default:
                    throw new Error("Unknown type: " + type);
            }
        };
        Dialog.prototype.isOpen = function() {
            return this.dialogArea && this.dialogArea.dialog("isOpen");
        };
        Dialog.prototype.open = function() {
            this.dialogArea = createDialogArea();
            this.updateDialogContentHTML();

            var deferred = $.Deferred();
            this.dialogArea.on("dialogopen", this, function(event) {
                deferred.resolveWith(event.data);
            });

            this.dialogArea.dialog(this.configuration);
            return deferred.promise();
        };
        Dialog.prototype.updateDialogContentHTML = function(html) {
            if(html) {
                this.dialogHTML = html;
            }
            this.dialogArea.html(this.dialogHTML);
        };

 //alert("dialog B");
       return {
            smartFormExitDialog : function(options) {
                var dialog = new Dialog({
                    type : "confirm",
                    text : "Changes were made to the form. What would you like to do?",
                    buttons : [
                        {
                            text: "Save Changes & Exit",
                            callbackFunction : function() {
                                this.close();
                                if(options.saveFunction) {
                                    options.saveFunction.apply(this);
                                }
                            }
                        },
                        {
                            text : "Ignore Changes & Exit",
                            callbackFunction : function() {
                                this.close();
                                if(options.ignoreFunction) {
                                    options.ignoreFunction.apply(this);
                                }
                            }
                        },
                        {
                            text : "Cancel",
                            callbackFunction : function() {
                                this.close();
                                if(options.cancelFunction) {
                                    options.cancelFunction.apply(this);
                                }
                            }
                        }
                    ]
                });
                return dialog.open();
            },
            alert : function(text, options) {
                var dialog = new Dialog({
                    type : "alert",
                    text : text,
                    buttons : [
                        {
                            text : "OK",
                            callbackFunction : function() {
                                this.close();
                                if(options && options.callbackFunction) {
                                    options.callbackFunction.apply(this);
                                }
                            }
                        }
                    ]
                });
                return dialog.open();
            },
            confirm : function(text, options) {
                var dialog = new Dialog({
                    type : "alert",
                    text : text,
                    buttons : [
                        {
                            text : "OK",
                            callbackFunction : function() {
                                this.close();
                                if(options && options.callbackFunctions && options.callbackFunctions.ok) {
                                    options.callbackFunctions.ok.apply(this);
                                }
                            }
                        },
                        {
                            text : "Cancel",
                            callbackFunction : function() {
                                this.close();
                                if(options && options.callbackFunctions && options.callbackFunctions.cancel) {
                                    options.callbackFunctions.cancel.apply(this);
                                }
                            }
                        }

                    ]
                });
                return dialog.open();
            },
            open : function(options) {
                var dialog = new Dialog(options);
                return dialog.open();
            }
        };
    });