//    ATTN:
//     _____              _ _     _    _            _______ _     _
//    |  __ \            ( ) |   | |  | |          |__   __| |   (_)
//    | |  | | ___  _ __ |/| |_  | |  | |___  ___     | |  | |__  _ ___
//    | |  | |/ _ \| '_ \  | __| | |  | / __|/ _ \    | |  | '_ \| / __|
//    | |__| | (_) | | | | | |_  | |__| \__ \  __/    | |  | | | | \__ \
//    |_____/ \___/|_| |_|  \__|  \____/|___/\___|    |_|  |_| |_|_|___/
//
//    This module is not finalized. The functionality contained herein
//    is not finalized. Everything in this file will change in future
//    releases. If you use this module or code, you are guaranteeing
//    the code you write will break in future releases.
//
//    To reiterate: DO NOT USE THIS MODULE.
//                  DO NOT USE THIS CODE.
//

/* globals define */

//
// By defining the globals explicitly, we are hoping to discourage
// the use of globals. We also hope this will be useful to help
// find and remove globals when the opportunity arises. Finally,
// we are adding this here so our View object can remain
// theoretically globals free.
//
define(
    [
        "underscore",
        "DebugUtil"
    ],
    function(
        _,
        DebugUtil
    ) {
        "use strict";
//alert("global A");

        var DEBUG = new DebugUtil();

        var global = window;

        var huntGlobals = false;

        function logDeprecatedMessage(variableString, whatToDo) {
            if(!DEBUG.isLoggingOn()) {
                return;
            }
            var str = "Global Object '"+ variableString + "' was used." +
                (whatToDo ? "\nInstead use: " + whatToDo : "");

            if(huntGlobals) {
                throw new Error(str);
            } else {
                DEBUG.warn(str);
            }
        }

        function logObsoleteMessage(variableString, whatToDo) {
            if(!DEBUG.isLoggingOn()) {
                return;
            }
            var str = "Obsolete Global Object '"+ variableString + "' was used. Functionality will removed in a future version." +
                (whatToDo ? "\nInstead use: " + whatToDo : "");

            if(huntGlobals) {
                throw new Error(str);
            } else {
                DEBUG.warn(str);
            }
        }



        // We use a function to grab the variable because we cannot
        // Guarantee the global variables won't change from external scripts
//alert("global B");

        return {
            show : function() {
                require(["detect-global"], function(detect) {
                    var props = detect();
                    function getPropsCount(object) {
                        var count = 0;
                        for (var prop in object) {
                            count++;
                        }
                        return count;
                    }
                    DEBUG.log("found " + getPropsCount(props), props);
                });
            },
            hunt : function(enabled) {
                huntGlobals = enabled || true;
            },
            get: function(variableString) {
                logDeprecatedMessage(variableString);
                return global[variableString];
            },
            set : function(variableString, value) {
                logDeprecatedMessage(variableString);
                /*jshint -W093 */
                return global[variableString] = value;
                /*jshint +W093 */
            },
            getFunction: function(func, variableString, whatToDo) {
                return function() {
                    logDeprecatedMessage(variableString, whatToDo);
                    return func.apply(this, arguments);
                }
            },
            obsoleteFunction: function(variableString, whatToDo) {
                return logObsoleteMessage(variableString, whatToDo);
            }
        };

    });
